const express = require('express');

const idhmeApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const id2str = require(`${libs}/middlewares/id2str`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const config = require(`${libs}/config`); 

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

idhmeApp.use(cache('15 day'));

idhmeApp.get('/year_range', (req, res, next) => {
    req.sql.from('adh_idh')
    .field('MIN(adh_idh.ano_censo)', 'start_year')
    .field('MAX(adh_idh.ano_censo)', 'end_year')
    .where('adh_idh.ano_censo > 2014');
    next();
}, query, (req, res, next) => {
    req.sql.from('adh_idh_uf')
    .field('MIN(adh_idh_uf.ano_censo)', 'start_year')
    .field('MAX(adh_idh_uf.ano_censo)', 'end_year')
    .where('adh_idh_uf.ano_censo > 2014');
    req.old_result = req.result;
    next();
}, query, (req, res, next) => {
    // console.log(req.old_result[0].start_year);
    // console.log(req.result[0].start_year);
    if (req.old_result[0].start_year < req.result[0].start_year) {
        req.result[0].start_year = req.old_result[0].start_year;
    }
    if (req.old_result[0].end_year > req.result[0].end_year) {
        req.result[0].end_year = req.old_result[0].old_result;
    }
    next();
}, query, response('range'));

idhmeApp.get('/years', (req, res, next) => {
    req.sql.from('adh_idh')
    .field('DISTINCT adh_idh.ano_censo', 'year')
    .where('adh_idh.ano_censo > 2014');
    next();
}, query, (req, res, next) => {
    req.oldResult = req.result;

    req.sql = squel.select();

    req.sql.from('adh_idh_uf')
    .field('DISTINCT adh_idh_uf.ano_censo', 'year')
    .where('adh_idh_uf.ano_censo > 2014');;
    next();
}, query, (req, res, next) => {
    let result = Object.assign(req.oldResult, req.result);
    req.result = result;
    next();
}, response('years'));

idhmeApp.get('/source', (req, res, next) => {
    req.sql.from('fonte')
    .field('fonte', 'source')
    .where('tabela = \'adh_idh\'');
    next();
}, query, response('source'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id',
        table: 'adh_idh'
    },
    join: {
        primary: 'id',
        foreign: 'municipio_id',
        foreignTable: 'adh_idh'
    }
}).addValue({
    name: 'min_year',
    table: '@',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        table: '@',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: '@',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        table: '@',
        field: 'ano_censo'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id',
        table: '@'
    },
    join: {
        primary: 'id',
        foreign: 'estado_id',
        foreignTable: '@'
    }
});

idhmeApp.get('/', rqf.parse(), (req, res, next) => {

    if (("city" in req.dims) || ("city" in req.filter)) {
        req.sql.from('adh_idh')
        .field('adh_idh.idhm_e', 'total')
        .field('adh_idh.ano_censo', 'year')
        .field('adh_idh.municipio_id', 'city_id')
        .group('adh_idh.idhm_e')
        .group('adh_idh.ano_censo')
        .group('adh_idh.municipio_id');
    } else if (("state" in req.filter) || ("state" in req.dims)) {
        req.sql.from('adh_idh_uf')
        .field('adh_idh_uf.idhm_e', 'total')
        .field('adh_idh_uf.ano_censo', 'year')
        .field('adh_idh_uf.estado_id', 'state_id')
        .group('adh_idh_uf.idhm_e')
        .group('adh_idh_uf.ano_censo')
        .group('adh_idh_uf.estado_id');
    } else {
        res.status(400);
        next({
            status: 400,
            message: 'Wrong/No filter specified'
        });
    }
    next();
}, rqf.build(), query, id2str.transform(), response('idhme'));

module.exports = idhmeApp;
